function [L,dl,ddl]=loglike(X,b,cens)
    
    obs = size(X,1);
    freq   = ones(obs,1);
    atrisk = (1:obs)';
    tied   = zeros(obs,1);  

    % Compute log likelihood L
    obsfreq = freq .* cens;
    Xb = X*b;
    r = exp(Xb);
    risksum = flipud(cumsum(flipud(freq.*r)));
    risksum = risksum(atrisk);
    L = obsfreq'*(Xb - log(risksum));
    
    if nargout>=2
        % Compute first derivative dL/db
        [n,p] = size(X);
        Xr = X .* repmat(r,1,p);
        Xrsum = flipud(cumsum(flipud(Xr)));
        Xrsum = Xrsum(atrisk,:);
        A = Xrsum ./ repmat(risksum,1,p);
        dl = obsfreq' * (X-A);
        if nargout>=5
            % Set the mlflag (monotone likelihood flag) to indicate if the
            % likelihood appears to be monotone, not at an optimum.  This
            % can happen if, at each of the sorted failure times, the
            % specified linear combination of the X's is larger than that
            % of all other observations at risk.
            if n>2
                mlflag = all(cens(1:end-1) | (diff(Xb)<0 & ~tied(1:end-1)));
            else
                mlflag = true;
            end
        end
    end
    if nargout>=3
        % Compute second derivative d2L/db2
        t1 = repmat(1:p,1,p);
        t2 = sort(t1);
        XXr = X(:,t1) .* X(:,t2) .* repmat(r.*freq,1,p^2);
        XXrsum = flipud(cumsum(flipud(XXr),1));
        %XXrsum = XXrsum(atrisk,:,:) ./ repmat(risksum,[1,p^2]); % original
        XXrsum = XXrsum(atrisk,:) ./ repmat(risksum,[1,p^2]);  % changed
        ddl = reshape(-obsfreq'*XXrsum, [p,p]);
        ddl = ddl + A'*(A.*repmat(obsfreq,1,p));
    end
    
end
