function [bta] = DegreeNetCox(Data, lambda, d, L)

% Data: a m-by-n matrix, m is the number of genes, n is the number of
% samples. Note: the sample dimesion of Data is sorted by the survival time (Times)
% d: censor information, 1 is uncensored, 0 is censored.
% S: normalized network information

% m: number of genes
% n: number of samples
[m,n] = size(Data);

d = double(d);

%maximum interation for the outer loop
maxIter1 = 10;

%maximum iteration for the inner loop (most of time it less than 10 iterations)
maxIter2 = 1000;

%gamma = zeros(length(lamda),1,n);
bta   = zeros(length(lambda),1,m);

inner_conv_val = 1e-8;
outer_conv_val = 1e-6;

Z = Data'/L*Data;

for mm = 1:length(lambda) % for every lambda
    warning 'off';
    fprintf('  going to calculate for lambda=%E\n',lambda(mm));
    gamma_outer = rand(n,1);         % random start ??
    lp          = zeros(maxIter1,1); % log-partial likelihood
    pena        = zeros(maxIter1,1); % penalization

    C = Z * gamma_outer;

    M = zeros(n,n);
    for j = 1:n % for every n number of samples
        M(j,:) = C' - C(j);
    end
    M = exp(M);

    tmp = zeros(n,1);
    for k = 1:n
        IX = find(d(1:k)==1);
        for p = 1:length(IX)
            tmp(k) = tmp(k) + 1/sum(M(k,IX(p):n));
        end
    end

    lnh = zeros(n,1);
    for p = 1:n-1
        lnh(p) = -C(p) - log(1 + sum(M(p,p+1:n)));
    end
    lnh(n) = -C(n);

    lp_old = 0;

    for i = 1:maxIter1
        gamma_outer_old = gamma_outer;

        % penalization
        pena(i) = (1/2)*lambda(mm)*(gamma_outer'*Z*gamma_outer);
        % log-partial likelihood
        lp(i) = sum(-tmp + d.*(lnh + C)) - pena(i);

        gamma_inner = gamma_outer;
        lp_inner_old = 0;

        % Newton-Raphson
        for t = 1:maxIter2
            gamma_inner_old = gamma_inner;
            dta = d -tmp;
            if any(isnan(dta))
                continue;
            end
            % first derivative
            firstD = Z*dta - lambda(mm)*Z*gamma_inner;
            D = diag(tmp); % diagonal matrix
            % second derivative
            secondD = -Z*D*Z'- lambda(mm)*Z;
            gamma_inner = gamma_inner - secondD \ firstD;
            tmp = tmp.*exp(Z*gamma_inner-Z*gamma_inner_old);

            pena_inner = (1/2)*lambda(mm)*(gamma_inner'*Z*gamma_inner);
            lp_inner = sum(-tmp + d.*(lnh + C)) - pena_inner;

            % convergence check
            if (max(abs(gamma_inner-gamma_inner_old)) < inner_conv_val  || abs(lp_inner-lp_inner_old) < inner_conv_val )
                break;
            end

            lp_inner_old = lp_inner;

            if t == 1000
                fprintf('    DegreeNetCox: The Newton-Raphson is not converge, lambda=%E\n',lambda(mm))
            end

        end % Newton-Raphson
        gamma_outer = gamma_inner;

        % if converge then break
        if (max(abs(gamma_outer-gamma_outer_old)) < outer_conv_val || abs(lp(i)-lp_old) < outer_conv_val )
            break;
        end
        %update the baseline hazard
        lp_old = lp(i);

        C = Z * gamma_outer;

        M = zeros(n,n);
        for j = 1:n
            M(j,:) = C' - C(j);
        end
        M = exp(M);

        tmp = zeros(n,1);
        for k = 1:n
            IX = find(d(1:k)==1);
            for p = 1:length(IX)
                tmp(k) = tmp(k) + 1/sum(M(k,IX(p):n));
            end
        end
        lnh = zeros(n,1);
        for p = 1:n-1
            lnh(p) = -C(p) - log(1 + sum(M(p,p+1:n)));
        end
        lnh(n) = -C(n);
    end
    %gamma(mm,1,:) = gamma_outer;
    bta(mm,1,:) = L\Data*gamma_outer;
    %
    fprintf('    finished in %d iterations (lambda: %.3E, i: %d, t: %d) with loglike: %f | min bta = %f | max bta = %f\n',...
        i*maxIter1-maxIter1 + t,lambda(mm),i,t, loglike(Data', reshape(bta(mm,1,:),size(bta,3),1), d),...
	min(bta(mm,1,:)), max(bta(mm,1,:)));
end
